@@ -82,6 +82,7 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
         # to the next line
 
         self.tag_configure("current_line", background="#f2f2f2")
+        self.tag_raise("sel", "current_line")
 
         # since we automatically add a space after an inserted pipe,
         # we want to ignore a space that immediately follows since
@@ -110,6 +111,28 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
     def current_row_number(self):
         return self.index("insert").split(".")[0]
 
+    def find_start_of_statement(self, index="insert"):
+        '''This can be overridden by a subclass to do something more interesting
+
+        For example, when building an editor for a robotframework test
+        case, it could look up for the first line that is not a
+        continuation line
+        '''
+        return self.index("%s linestart" % index)
+
+    def find_end_of_statement(self, index="insert", mark="w00t"):
+        '''This can be overridden by a subclass to do something more interesting
+
+        For example, when building an editor for a robotframework test
+        case, it could look down for additional continuation lines.
+        '''
+        return self.index("%s lineend" % index)
+
+    def get_row_number(self, index):
+        '''Return the row number for a given index'''
+        linenum = int(self.index(index).split(".")[0])
+        return linenum
+
     def set_completion_choices(self, choices):
         self.list.delete(0, "end")
         for string in choices:
@@ -124,12 +147,6 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
         except:
             return "WTF?"
 
-    def select_similar_rows(self, index="insert"):
-        linenum = int(self.index("insert").split(".")[0])
-        rows = self.find_like_rows(linenum)
-        self.tag_remove("sel", 1.0, "end")
-        self.tag_add("sel", "%s.0" % rows[0], "%s.0 lineend" % rows[-1])
-
     def get_selected_rows(self):
         '''Return all of the data for all lines that contain the selection
 
@@ -180,7 +197,8 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
         '''
         lines = []
         for row in rows:
-            data = "| " + " | ".join(row)
+            data = " | ".join(row)
+            data = "|" + data if data.startswith(" ") else "| " + data
             lines.append(data.rstrip())
         return "\n".join(lines)
 
@@ -197,37 +215,38 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
         self.tag_add("sel", "sel_start", "sel_end")
         self.mark_unset("sel_start")
         
-    def find_like_rows(self, linenumber):
-        '''Find rows with the same number of columns
+    def get_current_statement(self):
+        
+        '''Return the current statement as a list of cells
 
-        Remarkably, this is pretty fast. About 1ms the last time I timed it
-
-        Admittedly, the algorithm is pretty clunky and could stand for some optimization
+        This doesn't do it exactly like robot does, but it's
+        close enough for now. 
         '''
-        start = end = anchor = int(linenumber)
-        row = self.get_row(linenumber)
-        if len(row) == 1 and row[0] == "":
-            self.tag_remove("like", "1.0", "end")
-            self.tag_add("like", "%s.0 linestart" % linenumber, "%s.0 lineend+1c" % linenumber)
-            return (anchor,)
-        required_rows = len(row)
-        for linenumber in range(anchor-1, 0, -1):
-            row = self.get_row(linenumber)
-            if len(row) != required_rows or (len(row) == 1 and row[0] == ""):
-                break
-            start = linenumber
+        text = self.get_current_statement_text()
+        if not text.lstrip().startswith("|"):
+            text = "| " + text
+        lines = text.split("\n")
+        statement = self._split_row(lines[0])
+        for line in lines[1:]:
+            # robot requires a leading pipe; make sure there is one,
+            # otherwise it will split on whitespace. Yuck.
+            
+            line = line.strip(" ")
+            if not line.startswith("|"):
+                line = "| " + line
+            row = self._split_row(line)
+            # we're relying on the fact that in standard robot
+            # format for continuation lines, the first cell will
+            # be blank, and the next cell will contain "..."
+            statement.extend(row[2:])
+        return statement
 
-        lastline = int(self.index("end").split(".")[0])
-        for linenumber in range(anchor+1, lastline):
-            row = self.get_row(linenumber)
-            if len(row) != required_rows or (len(row) == 1 and row[0] == ""):
-                break
-            end = linenumber
+    def get_current_statement_text(self):
+        start = self.find_start_of_statement()
+        end = self.find_end_of_statement()
+        text = self.get(start, end)
+        return text
 
-        self.tag_remove("like", "1.0", "end")
-        self.tag_add("like", "%s.0 linestart" % start, "%s.0 lineend+1c" % end)
-        return range(start, end+1)
-    
     def get_lines(self, start="1.0", end="end"):
         '''Generator which returns all whole lines from start to end
         
@@ -246,7 +265,8 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
         return result
 
     def get_row(self, linenumber):
-        '''Return a row as a list of values
+        '''Return a row as a list of values. The values will be 
+           stripped of leading and trailing whitespace
 
         This replicates the functionality of robot.parsing.txtreader.split_row
         in robot 2.7 (which isn't officially out yet...)
@@ -373,7 +393,6 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
             return "break"
 
     def _on_triple_click(self, event):
-      print "on_triple_click..."
       self.select_current_cell()
       return "break"
 
@@ -468,8 +487,6 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
             self.edit_separator()
             return "break"
 
-    ### auto complete stuff
-
     def _on_click(self, event):
         try:
             if self.complete_frame.winfo_viewable():
@@ -492,7 +509,6 @@ class DynamicTableEditor(tk.Text, HighlightMixin):
             self.event_generate("<<AutoComplete>>")
             
         line = self.index("insert").split(".")[0]
-        self.find_like_rows(line)
 
     def _on_any_key(self, event):
         if self.complete_frame.winfo_viewable():
